#include "dcastaway.h"

#include "menu.h"

#include "vkbd.h"
#include "keymap.h"
#include "config.h"
#include "st.h"
#include "mem.h"
#include "cpu/68000.h"
#include "sound/sound.h"

#ifdef GP2X
#include "gp2xsys.h"
int up, down, left, right, upleft, upright, downleft, downright;
int joy2x[8];
#endif

static SDL_Joystick *jstick[2]={NULL,NULL};
extern int mainMenu_frameskip;
extern int mainMenu_sound;
extern int maxframeskip;
extern int mouse_mul;

#ifdef DREAMCAST
extern int  __sdl_dc_emulate_keyboard;
extern int  __sdl_dc_emulate_mouse;
#endif

extern char *dcastaway_image_file;
extern char *dcastaway_image_file2;

extern int fdc_commands_executed;
extern Uint32 autoframeskip_next;

extern int mainMenu_savedisk;

int changed_fdc0;
int changed_fdc1;

#define JOY_UP       1
#define JOY_DOWN     2
#define JOY_LEFT     4
#define JOY_RIGHT    8
#define JOY_BUTTON 128

void goMenu(void){
	int exitmode;
	int autosave=mainMenu_savedisk;

#ifdef DEBUG_FRAMESKIP
	print_frameskip();
#endif
	changed_fdc0=0;
	changed_fdc1=0;
    	audio_stop();
	vkbd_quit();
	render_no_border();
	init_text(0);
	if (!mainMenu_sound)
		SDL_PauseAudio(0);
	exitmode=run_mainMenu();
	if ((!changed_fdc0)&&(!changed_fdc1))
		menu_unraise();
	quit_text();
	maxframeskip=mainMenu_frameskip;
#ifdef DREAMCAST
        __sdl_dc_emulate_keyboard=0;
#endif
	if (changed_fdc0)
		FDCInit(0);
	if (changed_fdc1)
		FDCInit(1);
	render_blank_screen();
	fdc_commands_executed=0;
	if (exitmode==2)
	{
		if (autosave!=mainMenu_savedisk)
		{
			FDCInit(0);
			FDCInit(1);
		}
//		emulating=0;
		MemReInit();
		IOInit();
  		Ikbd_Reset();
    		HWReset();
	}
        vkbd_init();
	render_down_screen();
	if (mainMenu_sound)
	{
		Sound_Update_VBL();
    		audio_init();
	}
    	else
		SDL_PauseAudio(1);
	events_init();
	reset_frameskip();
}

static int nowSuperThrottle=0, goingSuperThrottle=0;
static int back_mainMenu_sound=0;

static void goSuperThrottle(void)
{
	if (!nowSuperThrottle)
	{
		back_mainMenu_sound=mainMenu_sound;
		mainMenu_sound=0;
		maxframeskip=80;
		SDL_PauseAudio(1);
		nowSuperThrottle=1;
	}
}

static void leftSuperThrottle(void)
{
	if (nowSuperThrottle)
	{
		mainMenu_sound=back_mainMenu_sound;
		if (mainMenu_sound)
		{
			Sound_Update_VBL();
			audio_init();
		}
		maxframeskip=mainMenu_frameskip;
		reset_frameskip();
		nowSuperThrottle=0;
	}
}

static void swapDisk(void)
{
	char *tmp=(char *)calloc(1,128);
	strcpy(tmp,dcastaway_image_file);
	strcpy(dcastaway_image_file,dcastaway_image_file2);
	strcpy(dcastaway_image_file2,tmp);
	free(tmp);
	FDCInit(1);
	FDCInit(0);
}

void events_init(void)
{
     int it=20;
#ifdef DREAMCAST
    __sdl_dc_emulate_keyboard=0;
#endif
#ifdef GP2X
    up=down=left=right=upleft=upright=downleft=downright=0;
#endif
    SDL_JoystickEventState(SDL_ENABLE);
    if (jstick[0]==NULL)
 	   jstick[0]=SDL_JoystickOpen(0);
    if (jstick[1]==NULL)
 	   jstick[1]=SDL_JoystickOpen(1);
    while(it-->0)
    {
	    SDL_Event event;
	    while( SDL_PollEvent(&event) )
		    SDL_Delay(1);
	    SDL_Delay(10);
    }
}

void do_events (void) 
{
    static int display_vkbd;
    static int joystateold[2]={0,0};
    int hat,jsn=0,joystate[2];
    SDL_Event event;
    SDLKey sym;
    static Uint32 ticks=(Uint32)-1;
    Uint32 now=SDL_GetTicks();

    joystate[0]=joystateold[0];
    joystate[1]=joystateold[1];
    if (vkbd_mode)
    {
	if (draw_border)
		vkbd_redraw();
//	maybe_border=0;
    	if (now-ticks>100)
    	{
	    render_vkbd_background();
	    vkbd_key=vkbd_process();
	    ticks=now;
    	}
    }

    /* Just process events, if there */
    while( SDL_PollEvent(&event) ) {

        switch(event.type) {
	case SDL_JOYBUTTONDOWN:
		jsn=event.jbutton.which;
		switch(event.jbutton.button)
		{
#ifdef GP2X
			case GP2X_BUTTON_B:
			case GP2X_BUTTON_UP: 	if (vkbd_mode)
							vkbd_move |= VKBD_UP;
					 	else
							up=true;
						break;
			case GP2X_BUTTON_DOWN:  if (vkbd_mode)
							vkbd_move |= VKBD_DOWN;
					 	else
							down=true;
						break;
			case GP2X_BUTTON_LEFT:  if (vkbd_mode)
							vkbd_move |= VKBD_LEFT;
					 	else
							left=true;
						break;
			case GP2X_BUTTON_RIGHT: if (vkbd_mode)
							vkbd_move |= VKBD_RIGHT;
					 	else
							right=true;
						break;
			case GP2X_BUTTON_UPLEFT: if (vkbd_mode){
							vkbd_move |= VKBD_UP;
							vkbd_move |= VKBD_LEFT;
							}
					 	else	{
							upleft=true;
							}
						break;

			case GP2X_BUTTON_UPRIGHT: if (vkbd_mode){
							vkbd_move |= VKBD_UP;
							vkbd_move |= VKBD_RIGHT;
							}
					 	else	{
							upright=true;
							}
						break;
			case GP2X_BUTTON_DOWNLEFT: if (vkbd_mode){
							vkbd_move |= VKBD_DOWN;
							vkbd_move |= VKBD_LEFT;
							}
					 	else	{
							downleft=true;
							}
						break;
			case GP2X_BUTTON_DOWNRIGHT: if (vkbd_mode){
							vkbd_move |= VKBD_DOWN;
							vkbd_move |= VKBD_RIGHT;
							}
					 	else	{
							downright=true;
							}
						break;

			case GP2X_BUTTON_X: if (!vkbd_mode) joystate[0] |= JOY_BUTTON; break;
			//case GP2X_BUTTON_B: if (!vkbd_mode) up=true; break;			
			case GP2X_BUTTON_Y: if (!vkbd_mode) IkbdKeyPress(keysymToAtari[SDLK_ESCAPE]); break;
			case GP2X_BUTTON_A: if (!vkbd_mode) IkbdKeyPress(keysymToAtari[SDLK_SPACE]); break;
			case GP2X_BUTTON_L: if (!vkbd_mode) IkbdKeyPress(keysymToAtari[SDLK_1]); break;
			case GP2X_BUTTON_R: if (!vkbd_mode) IkbdKeyPress(keysymToAtari[SDLK_2]); break;
			case GP2X_BUTTON_VOLDOWN: if (!vkbd_mode) IkbdKeyPress(keysymToAtari[SDLK_3]); break;						
			case GP2X_BUTTON_VOLUP: if (!vkbd_mode) IkbdKeyPress(keysymToAtari[SDLK_4]); break;
#else			
			case 2:
				if (!vkbd_mode)
					joystate[jsn] |= JOY_BUTTON;
				break;
			case 6:
				if (!vkbd_mode)
				{
					if (vkbd_button2)
            					IkbdKeyPress(keysymToAtari[vkbd_button2]);
					else
						joystate[jsn] |= JOY_UP;
				}
				break;
#ifdef DREAMCAST
			case 5:
				if ((!__sdl_dc_emulate_mouse)&&(!vkbd_mode))
                			IkbdMousePress(2);
				break;
			case 1:
				if ((!__sdl_dc_emulate_mouse)&&(!vkbd_mode))
                			IkbdMousePress(1);
				break;

#endif
#endif
		}
		break;
	case SDL_JOYBUTTONUP:
		jsn=event.jbutton.which;
		switch(event.jbutton.button)
		{
#ifdef GP2X
			case GP2X_BUTTON_B:
			case GP2X_BUTTON_UP: 	if (vkbd_mode)
							vkbd_move &= ~VKBD_UP;
					 	else
							up=false;
						break;
			case GP2X_BUTTON_DOWN: 	if (vkbd_mode)
							vkbd_move &= ~VKBD_DOWN;
					 	else
							down=false;
						break;
			case GP2X_BUTTON_LEFT: 	if (vkbd_mode)
							vkbd_move &= ~VKBD_LEFT;
					 	else
							left=false;
						break;
			case GP2X_BUTTON_RIGHT: if (vkbd_mode)
							vkbd_move &= ~VKBD_RIGHT;
					 	else
							right=false;
						break;
			case GP2X_BUTTON_UPLEFT: if (vkbd_mode){
							vkbd_move &= ~VKBD_UP;
							vkbd_move &= ~VKBD_LEFT;
							}
					 	else	{
							upleft=false;
							}
						break;
			case GP2X_BUTTON_UPRIGHT: if (vkbd_mode){
							vkbd_move &= ~VKBD_UP;
							vkbd_move &= ~VKBD_RIGHT;
							}
					 	else	{
							upright=false;
							}
						break;
			case GP2X_BUTTON_DOWNLEFT: if (vkbd_mode){
							vkbd_move &= ~VKBD_DOWN;
							vkbd_move &= ~VKBD_LEFT;
							}
					 	else	{
							downleft=false;
							}
						break;
			case GP2X_BUTTON_DOWNRIGHT: if (vkbd_mode){
							vkbd_move &= ~VKBD_DOWN;
							vkbd_move &= ~VKBD_RIGHT;
							}
					 	else	{
							downright=false;
							}
						break;
			case GP2X_BUTTON_X: 	if (vkbd_mode)
							vkbd_move=VKBD_BUTTON;
					 	else
							joystate[0] &= ~JOY_BUTTON;
						break;
			//case GP2X_BUTTON_B: if (!vkbd_mode) up=false; break;
			case GP2X_BUTTON_Y: if (!vkbd_mode) IkbdKeyRelease(keysymToAtari[SDLK_ESCAPE]); break;
			case GP2X_BUTTON_A: if (!vkbd_mode) IkbdKeyRelease(keysymToAtari[SDLK_SPACE]); break;
			case GP2X_BUTTON_L: if (!vkbd_mode) IkbdKeyRelease(keysymToAtari[SDLK_1]); break;
			case GP2X_BUTTON_R: if (!vkbd_mode) IkbdKeyRelease(keysymToAtari[SDLK_2]); break;
			case GP2X_BUTTON_VOLDOWN: if (!vkbd_mode) IkbdKeyRelease(keysymToAtari[SDLK_3]); break;						
			case GP2X_BUTTON_VOLUP: if (!vkbd_mode) IkbdKeyRelease(keysymToAtari[SDLK_4]); break;
			case GP2X_BUTTON_START: goMenu(); break;
			case GP2X_BUTTON_SELECT: if (!vkbd_mode) {
							render_force_background();
							render_up_screen();
							vkbd_mode=1;
							vkbd_redraw();
							}
						  else {
						  	render_force_background();
						  	render_down_screen();
						  	vkbd_mode=0;	
						  	}
						break;
#else			
			case 2: if (vkbd_mode)
					vkbd_move=VKBD_BUTTON;
				else
					joystate[jsn] &= ~JOY_BUTTON;
				break;
			case 6:
				if (vkbd_mode)
					vkbd_move=VKBD_BUTTON2;
				else
					if (vkbd_button2)
            					IkbdKeyRelease(keysymToAtari[vkbd_button2]);
					else
						joystate[jsn] &= ~JOY_UP;
				break;
			case 3:
				goMenu();
				break;
#ifdef DREAMCAST
			case 5:
				if ((!__sdl_dc_emulate_mouse)&&(!vkbd_mode))
                			IkbdMouseRelease(2);
				break;
			case 1:
				if ((!__sdl_dc_emulate_mouse)&&(!vkbd_mode))
                			IkbdMouseRelease(1);
				break;

#endif
#endif
		}
		break;
	case SDL_JOYAXISMOTION:
		switch(event.jaxis.axis)
		{
			case 3:
				if ((event.jaxis.value<=16)&&(nowSuperThrottle))
					leftSuperThrottle();
				else
				if ((goingSuperThrottle)&&(!vkbd_mode))
					goSuperThrottle();
				else
				if ((!vkbd_mode)&&(event.jaxis.value>=16)&&(!nowSuperThrottle))
				{
					render_force_background();
					render_up_screen();
					vkbd_mode=1;
					vkbd_redraw();
				}
				break;
			case 2:
				if (vkbd_mode)
				{
					render_force_background();
					render_down_screen();
					vkbd_mode=0;
					goingSuperThrottle=0;
				}
				else
					goingSuperThrottle=(event.jaxis.value>16);
				break;
#ifndef DREAMCAST
			case 1:
				if (event.jaxis.value < -3200)
				{
					joystate[0] |= JOY_UP;
					joystate[0] &= ~JOY_DOWN;
				}
				else
				if (event.jaxis.value > 3200)
				{
					joystate[0] |= JOY_DOWN;
					joystate[0] &= ~JOY_UP;
				}
				else
				{
					joystate[0] &= ~JOY_UP;
					joystate[0] &= ~JOY_DOWN;
				}
				break;
			case 0:
				if (event.jaxis.value < -3200)
				{
					joystate[0] |= JOY_LEFT;
					joystate[0] &= ~JOY_RIGHT;
				}
				else
				if (event.jaxis.value > 3200)
				{
					joystate[0] |= JOY_RIGHT;
					joystate[0] &= ~JOY_LEFT;
				}
				else
				{
					joystate[0] &= ~JOY_LEFT;
					joystate[0] &= ~JOY_RIGHT;
				}
				break;
#endif
		}
		break;
	case SDL_JOYHATMOTION:
		jsn=event.jhat.which;
#ifdef DREAMCAST
                hat=15^(SDL_JoystickGetHat(jstick[jsn],event.jhat.hat));
#else
             	hat=SDL_JoystickGetHat(jstick[jsn],event.jhat.hat);
#endif
		if (vkbd_mode)
		{
			if (hat & SDL_HAT_LEFT)
				vkbd_move |= VKBD_LEFT;
			else
			{
				vkbd_move &= ~VKBD_LEFT;
				if (hat & SDL_HAT_RIGHT)
					vkbd_move |= VKBD_RIGHT;
				else
					vkbd_move &= ~VKBD_RIGHT;
			}
			if (hat & SDL_HAT_UP)
				vkbd_move |= VKBD_UP;
			else
				vkbd_move &= ~VKBD_UP;
				if (hat & SDL_HAT_DOWN)
					vkbd_move |= VKBD_DOWN;
				else
					vkbd_move &= ~VKBD_DOWN;
		}
		else
		{
			if (hat & SDL_HAT_LEFT)
				joystate[jsn] |= JOY_LEFT;
			else 
			{
				joystate[jsn] &= ~JOY_LEFT;
				if (hat & SDL_HAT_RIGHT)
					joystate[jsn] |= JOY_RIGHT;
				else
					joystate[jsn] &= ~JOY_RIGHT;
			}
			if (hat & SDL_HAT_UP)
				joystate[jsn] |= JOY_UP;
			else
			{
				joystate[jsn] &= ~JOY_UP;
				if (hat & SDL_HAT_DOWN)
					joystate[jsn] |= JOY_DOWN;
				else
					joystate[jsn] &= ~JOY_DOWN;
			}
		}
		break;
        case SDL_KEYDOWN:
#ifndef DREAMCAST
	    if (event.key.keysym.sym==SDLK_PAGEUP)
		    goSuperThrottle();
	    else
	    if (event.key.keysym.sym==SDLK_PAGEDOWN)
		    swapDisk();
	    else
	    if (event.key.keysym.sym!=SDLK_F12 && event.key.keysym.sym!=SDLK_F11)
#endif
            IkbdKeyPress(keysymToAtari[event.key.keysym.sym]);
            break;
        case SDL_KEYUP:
#ifndef DREAMCAST
	    if (event.key.keysym.sym==SDLK_F11)
		    goMenu();
	    else
	    if (event.key.keysym.sym==SDLK_F12)
		    video_fullscreen();
	    else
	    if (event.key.keysym.sym==SDLK_PAGEUP)
		    leftSuperThrottle();
	    else
#endif
            IkbdKeyRelease(keysymToAtari[event.key.keysym.sym]);
            break;
        case SDL_MOUSEMOTION:
            IkbdMouseMotion(event.motion.x*mouse_mul, event.motion.y*mouse_mul);
            break;
        case SDL_MOUSEBUTTONDOWN:
#ifdef DREAMCAST
		if (__sdl_dc_emulate_mouse)
		{
			if (!vkbd_mode)
			{

				if (event.button.button==5)
                			IkbdMousePress(2);
				else if (event.button.button==1)
                			IkbdMousePress(1);
			}
		}
		else
		if (!event.button.button)
#else
		if (event.button.button<=1)
#endif
			IkbdMousePress(2);
		else
			IkbdMousePress(1);
            break;
        case SDL_MOUSEBUTTONUP:
#ifdef DREAMCAST
		if (__sdl_dc_emulate_mouse)
		{
			if (!vkbd_mode)
			{
				if (event.button.button==5)
                			IkbdMouseRelease(2);
				else if (event.button.button==1)
                			IkbdMouseRelease(1);
			}
		}
		else
		if (!event.button.button)
#else
		if (event.button.button<=1)
#endif
			IkbdMouseRelease(2);
		else
			IkbdMouseRelease(1);
            break;
        case SDL_QUIT:
            exit(0);
        }
#ifdef GP2X
    int i;
    for (i=0; i<8; i++) joy2x[i]=0;
    joystate[0]&= ~JOY_UP; joystate[0]&= ~JOY_DOWN; joystate[0]&= ~JOY_LEFT; joystate[0]&= ~JOY_RIGHT;
    joy2x[0]=upleft || (upleft&&up) || (upleft&&left);
    joy2x[1]=up && !upleft && !upright;
    joy2x[2]=upright || (upright&&up) || (upright&&right);
    joy2x[3]=right && !upright && !downright;
    joy2x[4]=downright || (downright&&right) || (downright&&down);
    joy2x[5]=down && !downleft && !downright;
    joy2x[6]=downleft || (downleft&&down) || (downleft&&left);
    joy2x[7]=left && !downleft && !upleft;

    if (joy2x[0])
    	{
    	    joystate[0] |= JOY_UP;
    	    joystate[0] |= JOY_LEFT;
    	}
    if (joy2x[1])
    	{
    	    joystate[0] |= JOY_UP;
    	}
    if (joy2x[2])
    	{
    	    joystate[0] |= JOY_UP;
    	    joystate[0] |= JOY_RIGHT;
    	}
    if (joy2x[3])
    	{
    	    joystate[0] |= JOY_RIGHT;
    	}
    if (joy2x[4])
    	{
    	    joystate[0] |= JOY_DOWN;
    	    joystate[0] |= JOY_RIGHT;
    	}
    if (joy2x[5])
    	{
    	    joystate[0] |= JOY_DOWN;
    	}
    if (joy2x[6])
    	{
    	    joystate[0] |= JOY_DOWN;
    	    joystate[0] |= JOY_LEFT;
    	}
    if (joy2x[7])
    	{
    	    joystate[0] |= JOY_LEFT;
    	}
#endif
    }
    
    if (joystateold[0]!=joystate[0])
    {
	    if (!vkbd_mode)
	    	IkbdJoystickChange(0,joystate[0]);
	    joystateold[0]=joystate[0];
    }
    if (joystateold[1]!=joystate[1])
    {
	    if (!vkbd_mode)
	    	IkbdJoystickChange(1,joystate[1]);
	    joystateold[1]=joystate[1];
    }
    if (vkbd_key)
    {
	    if (vkbd_keysave==-1234567)
	    {
	    	vkbd_keysave=keysymToAtari[vkbd_key];
 	        IkbdKeyPress(keysymToAtari[vkbd_key]);
	    }
    }
    else
    if (vkbd_keysave!=-1234567)
    {
            		IkbdKeyRelease(vkbd_keysave);
			vkbd_keysave=-1234567;
    }

}

